package hib.inher.anno.two;

import java.io.File;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.criterion.Restrictions;
import org.hibernate.transform.ResultTransformer;
import org.hibernate.type.StandardBasicTypes;

public class TestTwo {
    public static void main(String[] args) {
        Logger.getLogger("org.hibernate").setLevel(Level.SEVERE);
        SessionFactory sf = new Configuration().configure(new File("/work/hib/hibernateconfig_annotwo.xml")).buildSessionFactory();
        Session s = sf.openSession();
        // get one
        System.out.println("2 x One:");
        Animal a1 = (Animal)s.load(Animal.class, 1);
        System.out.println(a1);
        Cat a2 = (Cat)s.load(Cat.class, 1);
        System.out.println(a2);
        // get all
        System.out.println("All:");
        @SuppressWarnings("unchecked")
		List<Animal> a3list = s.createQuery("SELECT a FROM Animal AS a").list();
        for(Animal a3 : a3list) {
            System.out.println(a3);
        }
        // get some
        System.out.println("2 x Some:");
        Query q = s.createQuery("SELECT a FROM Animal AS a WHERE name LIKE :name");
        q.setParameter("name", "K%", StandardBasicTypes.STRING);
        @SuppressWarnings("unchecked")
		List<Animal> a4list = q.list();
        for(Animal a4 : a4list) {
            System.out.println(a4);
        }
        @SuppressWarnings("unchecked")
		List<Animal> a5list = s.createCriteria(Animal.class).add(Restrictions.like("name", "K%")).list();
        for(Animal a5 : a5list) {
            System.out.println(a5);
        }
        // conversion
        System.out.println("Conversion:");
        Query qq = s.createQuery("SELECT a.id AS id,a.name AS name,a.litermilkperday AS litermilkperday FROM Animal AS a WHERE name LIKE :name");
        qq.setParameter("name", "K%", StandardBasicTypes.STRING);
        qq.setResultTransformer(new ResultTransformer() {
			private static final long serialVersionUID = 1L;
			@SuppressWarnings("rawtypes")
			@Override
            public List transformList(List lst) {
                return lst;
            }
            @Override
            public Object transformTuple(Object[] vals, String[] nams) {
                Map<String,Object> namval = new HashMap<String,Object>();
                for(int i = 0; i < nams.length; i++) {
                    namval.put(nams[i], vals[i]);
                }
                if(namval.containsKey("miceperday")) {
                    Cat res = new Cat();
                    res.setId((Integer)namval.get("id"));
                    res.setName((String)namval.get("name"));
                    res.setMiceperday((Integer)namval.get("miceperday"));
                    return res;
                } else if(namval.containsKey("litermilkperday")) {
                    Cow res = new Cow();
                    res.setId((Integer)namval.get("id"));
                    res.setName((String)namval.get("name"));
                    res.setLitermilkperday((Integer)namval.get("litermilkperday"));
                    return res;
                } else {
                    throw new RuntimeException("WTF");
                }
            }
        });
        @SuppressWarnings("unchecked")
		List<Animal> a6list = qq.list();
        for(Animal a6 : a6list) {
            System.out.println(a6);
        }
        // tricky conversion
        System.out.println("Tricky conversion:");
        Query qqq = s.createQuery("SELECT a.id AS id,a.name AS name,a.miceperday AS miceperday,a.litermilkperday AS litermilkperday FROM Animal AS a");
        qqq.setResultTransformer(new ResultTransformer() {
			private static final long serialVersionUID = 1L;
			@SuppressWarnings("rawtypes")
			@Override
            public List transformList(List lst) {
                return lst;
            }
            @Override
            public Object transformTuple(Object[] vals, String[] nams) {
                Map<String,Object> namval = new HashMap<String,Object>();
                for(int i = 0; i < nams.length; i++) {
                    if(vals[i] != null) {
                        namval.put(nams[i], vals[i]);
                    }
                }
                if(namval.containsKey("miceperday")) {
                    Cat res = new Cat();
                    res.setId((Integer)namval.get("id"));
                    res.setName((String)namval.get("name"));
                    res.setMiceperday((Integer)namval.get("miceperday"));
                    return res;
                } else if(namval.containsKey("litermilkperday")) {
                    Cow res = new Cow();
                    res.setId((Integer)namval.get("id"));
                    res.setName((String)namval.get("name"));
                    res.setLitermilkperday((Integer)namval.get("litermilkperday"));
                    return res;
                } else {
                    throw new RuntimeException("WTF");
                }
            }
        });
        @SuppressWarnings("unchecked")
		List<Animal> a7list = qqq.list();
        for(Animal a7 : a7list) {
            System.out.println(a7);
        }
        // SQL
        System.out.println("SQL:");
        SQLQuery sq = s.createSQLQuery("(SELECT id,name,1,miceperday,NULL FROM cat_two WHERE name LIKE :name) UNION (SELECT id,name,2,NULL,litermilkperday FROM cow_two WHERE name LIKE :name)");
        sq.setParameter("name", "K%", StandardBasicTypes.STRING);
        @SuppressWarnings("unchecked")
		List<Object[]> a8list = sq.list();
        for(Object[] row : a8list) {
        	Animal a8;
        	int typ = ((BigInteger)row[2]).intValue();
        	if(typ == 1) {
        		Cat res = new Cat();
        		res.setId((Integer)row[0]);
        		res.setName((String)row[1]);
        		res.setMiceperday((Integer)row[5]);
        		a8 = res;
        	} else if(typ == 2) {
        		Cow res = new Cow();
        		res.setId((Integer)row[0]);
        		res.setName((String)row[1]);
        		res.setLitermilkperday((Integer)row[4]);
        		a8 = res;
        	} else {
                throw new RuntimeException("WTF");
        	}
            System.out.println(a8);
        }
        //
        s.close();
    }
}
